@@ -1,65 +1,53 @@
-"""API router that allows documentation and nesting itself to make a 'hyperlinked' browsable router."""
-
-from typing import Dict, Union, Type, Callable
-
-from django.http.request import HttpRequest
-from django.http.response import HttpResponseBase
-from django.urls import re_path, include, path
-from django.urls.resolvers import URLResolver, URLPattern
+from typing import Any, Callable, Dict, List, Type, Union
 
+from django.urls import include, path, re_path
+from django.urls.resolvers import URLPattern, URLResolver
+from rest_framework.routers import DefaultRouter
+from rest_framework.schemas import DefaultSchema
 from rest_framework.urlpatterns import format_suffix_patterns
-from rest_framework.routers import DefaultRouter, APIRootView
-from rest_framework.viewsets import ViewSetMixin, ViewSet
 from rest_framework.views import APIView
+from rest_framework.viewsets import ViewSet, ViewSetMixin
+
+from .views import APIRootView
 
 
 __all__ = ["APIRouter"]
 
 
-_urls_type = list[Union[URLResolver, URLPattern]]
-
-
-# Root view docstring becomes the browsable API description, so we overwrite it here
-class APIRoot(APIRootView):
-    """Welcome! This is the API root."""
+UrlsType = list[Union[URLResolver, URLPattern]]
 
 
 class APIRouter(DefaultRouter):
     """Router that will show APIViews in API root."""
 
-    root_view_name: str = "api-root"
-    APIRootView: APIRootView = APIRoot
+    def __init__(self, name: str = None, docstring: str = None, **kwargs):
+        self._navigation_routes: Dict[str, "APIRouter"] = {}
 
-    def __init__(self, *args, **kwargs):
-        self._navigation_routes: Dict[str, "DefaultRouter"] = {}
-        super(APIRouter, self).__init__(*args, **kwargs)
+        name = name if name is not None else "APIRootView"
+        self._root_view: Type[APIRootView] = type(name, (APIRootView,), {})  # noqa
+        self._root_view.__doc__ = docstring if docstring else APIRootView.__doc__
+        self._root_view.schema = DefaultSchema() if kwargs.get("show_in_shema", False) else None
+        self._root_view._ignore_model_permissions = kwargs.get("ignore_model_permissions", False)
+
+        self.root_view_name: str = name
+        super().__init__(**kwargs)
 
     @property
-    def navigation_routes(self) -> Dict[str, "DefaultRouter"]:
+    def navigation_routes(self) -> Dict[str, "APIRouter"]:
         """Add urls from these routers to this routers urls under the root-view of the added router,
         which will be named after the given key. This enables browser navigation of the API."""
         return self._navigation_routes
 
     @navigation_routes.setter
-    def navigation_routes(self, value: Dict[str, "DefaultRouter"]):
+    def navigation_routes(self, value: Dict[str, "APIRouter"]):
         self._navigation_routes = value
 
-    def format_root_view(self, name: str, docstring: str) -> None:
-        """Format the root view browsable API documentation.
-
-        :param name: Root display name. CamelCase or snake_case to separate words.
-        :param docstring: Used as view documentation.
-        """
-        new_root: APIRootView = type(name, (APIRootView,), {})  # noqa
-        new_root.__doc__ = docstring
-        self.APIRootView = new_root
-
     def get_routes(self, viewset: Type[Union[ViewSet, APIView]]):
         if issubclass(viewset, ViewSetMixin):
             return super().get_routes(viewset)
         return []
 
-    def get_api_root_view(self, api_urls: _urls_type = None) -> Callable[[HttpRequest, ...], HttpResponseBase]:
+    def get_api_root_view(self, api_urls: UrlsType = None) -> Callable[..., Any]:
         api_root_dict = {}
         list_name = self.routes[0].name
 
@@ -72,10 +60,10 @@ class APIRouter(DefaultRouter):
         for basename in self.navigation_routes:
             api_root_dict[fr"{basename}"] = basename
 
-        return self.APIRootView.as_view(api_root_dict=api_root_dict)
+        return self._root_view.as_view(api_root_dict=api_root_dict)
 
-    def get_urls(self) -> _urls_type:
-        urls = []
+    def get_urls(self) -> UrlsType:
+        urls: List[Union[URLResolver, URLPattern]] = []
         for prefix, view, basename in self.registry:
             if issubclass(view, ViewSetMixin):
                 continue
@@ -91,4 +79,3 @@ class APIRouter(DefaultRouter):
             urls.append(path(f"{basename}/", include(router.urls)))
 
         return super().get_urls() + urls
-
